#include <windows.h>
#include <gl/glu.h>
#include <stdio.h>
#include <math.h>

#define JAVAEXPORT  __declspec(dllexport) __stdcall
#define JENV        void *jenv, void *jclass
#define R		     0.7
#define H		     5.0

const GLfloat K = 360.0 / 20.0 * M_PI / 180.0;
const GLfloat RAD = M_PI / 180;

HGLRC hrc;
HDC hdc;
HHOOK id;
PAINTSTRUCT ps;

void SetDCPixelFormat(void);
void DrawCylinder(void);
void DrawSphere(GLfloat, int, int);
int created = 0;

void SetDCPixelFormat()
{
  static PIXELFORMATDESCRIPTOR pfd =
  {
    sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
    1,                                      // version number
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
    | PFD_DOUBLEBUFFER,                     // flags
    PFD_TYPE_RGBA,                          // RGBA pixel values
    16,                                     // 24-bit color
    0, 0, 0, 0, 0, 0,                       // don't care about these
    0, 0,                                   // no alpha buffer
    0, 0, 0, 0, 0,                          // no accumulation buffer
    16,                                     // 32-bit depth buffer
    0,                                      // no stencil buffer
    0,                                      // no auxiliary buffers
    PFD_MAIN_PLANE,                         // layer type
    0,                                      // reserved(must be 0)
    0, 0, 0                                 // no layer masks
  };
  int pixelFormat;

  pixelFormat = ChoosePixelFormat(hdc, &pfd);
  SetPixelFormat(hdc, pixelFormat, &pfd);
  DescribePixelFormat(hdc, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
}

LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam)
{
  CWPSTRUCT *c = (CWPSTRUCT *)lParam;

  if (nCode == HC_ACTION)
  {
    if (created == 1)
    {
      hdc = GetDC(c -> hwnd);
      SetDCPixelFormat();
      hrc = wglCreateContext(hdc);
      wglMakeCurrent(hdc, hrc);
      created++;
    }
    return 0;
  }
  return CallNextHookEx(id, nCode, wParam, lParam);
}

void JAVAEXPORT Java_Util_CreateOpenGL(JENV)
{
  if (!created)
  {
    id = SetWindowsHook(WH_CALLWNDPROC, (HOOKPROC)CallWndProc);
    created++;
  }
}

void JAVAEXPORT Java_Util_ReleaseOpenGL(JENV)
{
  wglMakeCurrent(NULL, NULL);
  wglDeleteContext(hrc);
  UnhookWindowsHookEx(id);
}

void JAVAEXPORT Java_Util_SwapBuffers(JENV)
{
  if (created)
    SwapBuffers(hdc);
}

void DrawCylinder()
{
  GLfloat x, y;
  int i = 1;

  glBegin(GL_QUAD_STRIP);
    glNormal3f(1.0, 0.0, 0.0);
    glVertex3f(R, 0.0, -H);
    glVertex3f(R, 0.0, H);
    for (; i <= 10; i++)
    {
      x = cos(i * K) * R;
      y = sin(i * K) * R;
      glNormal3f(x / R, 0, y / R);
      glVertex3f(x, y, -H);
      glVertex3f(x, y, H * 4);
    }
 glEnd();
}

void DrawSphere(GLfloat r, int slices, int segments)
{
  GLfloat old_x = r, old_y = 0, x, y;
  GLfloat C1 = 360.0 / (GLfloat)segments, C2 = 2.0 * r / (GLfloat)slices;
  GLfloat old_z = -r, z = -r, r2, old_r = 0;
  int i, j = -slices / 2;

    for (; j < slices / 2; j++)
    {
      z += C2;
      if (r * r - z * z < 0)
        r2 = 0;
      else
        r2 = sqrt(r * r - z * z);
      for (i = 0; i < segments + 1; i++)
      {
        x = cos(i * C1 * RAD);
        y = sin(i * C1 * RAD);
        glBegin(GL_QUADS);
//          glNormal3f(x, y, z);
          glVertex3f(x * r2, y * r2, z);
//          glNormal3f(old_x, old_y, z);
          glVertex3f(old_x * r2, old_y * r2, z);
//          glNormal3f(old_x, old_y, old_z);
          glVertex3f(old_x * old_r, old_y * old_r, old_z);
//          glNormal3f(x, y, old_z);
          glVertex3f(x * old_r, y * old_r, old_z);
        glEnd();
        old_x = x;
        old_y = y;
      }
      old_z = z;
      old_r = r2;
    }
}

void JAVAEXPORT Java_Util_DrawShot(JENV)
{
  DrawSphere(0.04, 10, 10);
}

void JAVAEXPORT Java_Util_DrawShotGun(JENV)
{
  GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
  GLfloat local_view[] = { 0.0 };
  GLfloat no_mat [] = { 0.0, 0.0, 0.0 };
  GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat mat_ambient[] = { 0.3, 0.3, 0.3, 1.0 };
  GLfloat high_shininess[] = { 50.0 };

  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glTranslatef(-0.7, -3.0, -16);
  glRotatef(7, 1.0, 0.0, 0.0);

  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, no_mat);

  DrawCylinder();
  glTranslatef(1.4, 0.0, 0.0);
  DrawCylinder();

  glDisable(GL_LIGHTING);
}

void JAVAEXPORT Java_Util_gluPerspective(JENV, GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar) { gluPerspective(fovy, aspect, zNear, zFar); }

